#!/usr/bin/perl

use strict;
use POSIX qw(strftime);
use Time::Local;

my $infile;
my $outfile;
my $CONF_C = 1.960; # 95% Konfidenz Intervall
#my $CONF_C = 1.645; # 90% Konfidenz Intervall
#my $CONF_C = 2.576; # 99% Konfidenz Intervall

my $drfactor = 1;

if ($#ARGV < 1) {
	print "USAGE: 	$0 INPUT-File OUTPUT-File <DATARATE FACTOR>\n";
        print "         DATARATE FACTOR: 1024 for KBits/s ...\n\n";
} else {

        $infile = $ARGV[0];
        $outfile = $ARGV[1];
        if ($#ARGV == 2) { $drfactor = $ARGV[2]; }

        if (!(-r $infile)) {
        	die ("Can't open $infile\n");
        }

        #printf ("Input File:        %s\n", $infile);
        #printf ("Output File:       %s\n", $outfile);


	my $line;
        my $hour;

        my %SUM;
        my %VALUES;
        my $valCnt;

	open (IN, "< $infile");

        while ($line = <IN>) {
        	chomp ($line);
#		printf ("%s\n", $line);

                if ($line =~ /^[0-9]/) {
                	# UDP
	                if ($line =~ /^(.{18})[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)/){
				$hour = get_hour($1);

				$SUM{$hour}->[0] += $3;		# Output Data Rate Bits/s
				$SUM{$hour}->[1] += $4;		# Input Data Rate Bits/s
				$SUM{$hour}->[2] += $5;		# Jitter
				$SUM{$hour}->[3] += $6;		# Packets failed
				$SUM{$hour}->[4] += $7;         # Packets transmitted
				$SUM{$hour}->[5] += 1;		# Count

                                push(@{$VALUES{$hour}}, [$3, $4, $5, $6, $7]);
                                $valCnt = 5;

	                }
                        # TCP
	                elsif ($line =~ /^(.{18})[\s|\t]+([\d|\.]+)[\s|\t]+([\d|\.]+)/){
				$hour = get_hour($1);

				$SUM{$hour}->[0] += $3;		# Output Data Rate Bits/s
				$SUM{$hour}->[5] += 1;		# Count

                                push(@{$VALUES{$hour}}, [$3]);
                                $valCnt = 1;
	                }

        	}

        } # END WHILE

        close (IN);

        my %MID;
        my %VAR;
        my %CONF_A;

        # Mittelwerte berechnen
        for (my $i=0; $i < 24; $i++) {
        	if (defined ($SUM{$i})) {
	                for (my $j=0; $j < $valCnt; $j++) {
	                        if (defined($SUM{$i}->[$j])) {
	                                $MID{$i}->[$j] = ($SUM{$i}->[$j]/$SUM{$i}->[5]);
	                                #printf ("%3d %3d %s\n", $i, $j, $MID{$i}->[$j]);
	                        }
	                }
                }
        }

        # Varianz / Standardabweichung berechnen
	for (my $i=0; $i < 24; $i++ ) {
        	if (defined(@{$VALUES{$i}})) {

                        # Summe fuer Varianz bilden
	        	foreach my $tmpVal (@{$VALUES{$i}}) {
                        	for (my $j=0; $j < $valCnt; $j++) {
					$VAR{$i}->[$j] += ($tmpVal->[$j] - $MID{$i}->[$j])**2;
#					printf ("%5d %5d %s\n", $i, $j, $VAR{$i}->[$j]);
                                }
                	}

			for (my $j=0; $j < $valCnt; $j++) {
                        	$VAR{$i}->[$j] = (1/($SUM{$i}->[5]-1))* $VAR{$i}->[$j];
                                $CONF_A{$i}->[$j] = ($CONF_C * (sqrt($VAR{$i}->[$j])) / sqrt($SUM{$i}->[5]));
                        }
                }

        }

        open (OUT, "> $outfile");

        printf OUT ("# Columns\n");
        printf OUT ("# 1. Hour of Day\n");
        printf OUT ("# 2. Output Datarate [Bits/second] Middle\n");
        printf OUT ("# 3. Output Datarate [Bits/second] Standard Dev.\n");
        printf OUT ("# 4. Output Datarate [Bits/second] Confidence Int. 'a'\n");
        printf OUT ("# 5. Input Datarate [Bits/second] Middle (Only UDP!)\n");
        printf OUT ("# 6. Input Datarate [Bits/second] Standard Dev. (Only UDP!)\n");
        printf OUT ("# 7. Input Datarate [Bits/second] Confidence Int. 'a' (Only UDP!)\n");
        printf OUT ("# 8. Jitter [ms] Middle (Only UDP!)\n");
        printf OUT ("# 9. Jitter [ms] Standard Dev. (Only UDP!)\n");
        printf OUT ("#10. Jitter [ms] Confidence Int. 'a' (Only UDP!)\n");
        printf OUT ("#11. Packets failed Middle (Only UDP!)\n");
        printf OUT ("#12. Packets failed Standard Dev. (Only UDP!)\n");
        printf OUT ("#13. Packets failed Confidence Int. 'a' (Only UDP!)\n");
        printf OUT ("#14. Total Packets Middle (Only UDP!)\n");
        printf OUT ("#15. Total Packets Standard Dev. (Only UDP!)\n");
        printf OUT ("#16. Total Packets Confidence Int. 'a' (Only UDP!)\n");
        printf OUT ("#\n");


 #       printf ("#%-3s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s\n",
#                "HOUR", "Output Datarate", "Middle Out.", "Std.Dev. Out.", "Conf. Int. a Out.",
#                "Input Datarate", "Middle In.", "Std.Dev. Int.", "Conf. Int. a In.", "Jitter", "Packets Failed", "Packets Sum"));
#
        for (my $i=0; $i<24; $i++) {
                if (defined($VAR{$i})) {
                        # Stunde
                        printf OUT ("%-4d", $i);

                        for (my $j=0; $j < $valCnt; $j++) {
                                if (defined($VAR{$i}->[$j])) {

                                	if ($j == 0 || $j == 1) {
                                        	$MID{$i}->[$j] = ($MID{$i}->[$j]/$drfactor);
                                        	$VAR{$i}->[$j] = ($VAR{$i}->[$j]/$drfactor);
                                        	$CONF_A{$i}->[$j] = ($CONF_A{$i}->[$j]/$drfactor);
                                        }

#                                        printf OUT ("%-14.2f %-14.2f %-14.2f %-14.2f ", $MID{$i}->[$j], sqrt($VAR{$i}->[$j]), ($MID{$i}->[$j]-$CONF_A{$i}->[$j]), ($MID{$i}->[$j]+$CONF_A{$i}->[$j]));
                                        printf OUT ("%-14.2f %-14.2f %-14.2f ", $MID{$i}->[$j], sqrt($VAR{$i}->[$j]), $CONF_A{$i}->[$j]);

#                                        printf ("X: %2d  %2d  ", $i, $j);
#                                        printf ("M:%20.4f  ", $MID{$i}->[$j]);
#                                        printf ("V:%20.4f  ", $VAR{$i}->[$j]);
#                                        printf ("D:%20.4f  ", sqrt($VAR{$i}->[$j]));
#                                        printf ("C:%20.4f  ", $CONF_A{$i}->[$j]);
#                                        printf ("\n");
                                }
                        }
                        # Zeilenende
                        printf OUT ("\n");
                }
        }

	close (OUT);

} # END Script
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub get_hour {
	# Input "YYYYMMDD HH:MM:SS"
	return sprintf("%d", substr($_[0], 9,2));
} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub convert_str2localtime {

	my $ticks;
        my $str = $_[0];
        $str =~ s/[-|:|\s]//g;
	$str =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/;
        my $year = $1-1900;
        my $month = $2-1;
        my $mday = $3;
        my $hour = $4;
        my $min = $5;
        my $sec = $6;

	$ticks = timelocal($sec, $min, $hour, $mday, $month, $year);
        return $ticks;

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub convert_localtime2str {
 	my @date = localtime(time);
        my $str;

      	if (defined $_[0]) { @date = localtime($_[0])}

        $str = sprintf("%d%02d%02d %02d:%02d:%02d", (1900+$date[5]), $date[4]+1, $date[3], $date[2], $date[1], $date[0]);

        return $str;

} # END SUB