#!/usr/bin/perl
# Usage: testbench.pl JOBDIR WORKDIR OUTPUTDIR

use strict;
use POSIX;
use POSIX qw(strftime);
use Time::Local;

#my $IPERF_CMD = 'C:\iperf\iperf.exe';
my $IPERF_CMD = 'C:\iperf\iperf.exe -l500';

my $JOBDIR;
my $WORKDIR;
my @jobs;
my %ACTIVE_JOB;

my $sleep_job_sec = 30;

my $sleep_error_start = 20;
my $sleep_error_backoff = 20;
my $sleep_error_max = 60;
my $sleep_error = $sleep_error_start;

my $idle = 0;
my $break_sig = 0;

# script to parse iperf output
# need args: inputfile outputfile datarate factor
my $PARSE_CMD='C:\iperf\parse2plot.pl';
my $PARSE_TIME=900;
my $PARSE_LASTTIME;
my $PLOT_CMD='C:\iperf\generatePlot.pl';


if ($#ARGV < 0) {
	print "USAGE: $0 <JOBDIR>\n\n";
} else {

        if ($ARGV[0] =~ /\\$/) { chop($ARGV[0]); }
        $JOBDIR = $ARGV[0];

        printf ("\nChecking Jobdir every %d seconds!\n\n", $sleep_job_sec);

        # Jobs suchen und verarbeiten
        do {
                # Auftrge aus Verzeichnis lesen
                # vorher @jobs leeren.
                undef @jobs;
                if (read_jobs() > 0) {
	                if ($break_sig != 0) {
	                        printf("FOUND 'BREAK' File. Stopped.\n");
	                        last;
	                }
                }

                if ($#jobs >= 0) {

                	# Job stoppen (falls vorhanden), wenn stoptime erreicht ist!
                        if ( defined(%ACTIVE_JOB) ) {
                        	if ( time >= $ACTIVE_JOB{"stoptime"} ) {
                                 	undef %ACTIVE_JOB;
	                         	$idle = 1;
                                }

                        }

                	# Wenn zurzeit ein Job ausgefhrt wird, keinen neuen Job aus der Liste holen!
                        if ( !((defined (%ACTIVE_JOB)) && ($idle == 0)) ) {
	                        print_job_details();
                             	%ACTIVE_JOB = get_next_job();
                        }

                        # Job ausfuehren, wenn Startzeit erreicht
                        if ($ACTIVE_JOB{"starttime"} <= time) {
                                do_job(%ACTIVE_JOB);
                                $idle = 0;
                        } else {
                        	# Zurzeit kein aktiver Job....
                        	printf ("[%s] Next Job \"%s\" begins at %s!\n", convert_localtime2str(time),$ACTIVE_JOB{"_filename"}, convert_localtime2str($ACTIVE_JOB{"starttime"}));
                                $idle = 1;
                        }

                } else {
                		# Keine Jobs in der Liste...
                        	printf ("[%s] No Jobs found.\n", convert_localtime2str(time));
                                $idle = 1;
                }

        	if ($idle != 0) {
                	# Solange nichts zu tun ist
                        # Alle $sleep_job_sec Sekunden Verzeichnisse kontrollieren.

                        if ( (($ACTIVE_JOB{"starttime"} - time) >= 0) &&  (($ACTIVE_JOB{"starttime"} - time) < $sleep_job_sec) ) {
	                        printf ("SLEEP TIME: %d\n", ($ACTIVE_JOB{"starttime"} - time));
                        	sleep($ACTIVE_JOB{"starttime"} - time);
                        } else {
	                        printf ("SLEEP TIME: %d\n", $sleep_job_sec);
	                	sleep($sleep_job_sec);
                        }
                }

        } while ($break_sig == 0);

}

# END OF PERL SCRIPT
# SUBROUTINES
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #

sub do_job {

        my $cmdline;
        my $outfileraw;
        my $outfiledata;
        my $outfileparse;
        my $outdir_plots;

        my $line;
        my $out_line;

        my @date = localtime(time);
        my $strHeader;
        my $result = 0;

        # Gibts wirklich einen aktiven job?
 	if (defined(%ACTIVE_JOB)) {

        	my $dateNow = get_date();

                $outfileraw = sprintf("%s\\RAW_%s_%s", $ACTIVE_JOB{"dir_raw"}, $dateNow, $ACTIVE_JOB{"_filename"});
                $outfileparse = sprintf("%s\\PAR_%s_%s", $ACTIVE_JOB{"dir_parse"}, $dateNow, $ACTIVE_JOB{"_filename"});
                $outfiledata = sprintf("%s\\DAT_%s_%s", $ACTIVE_JOB{"dir_plot_data"}, $dateNow, $ACTIVE_JOB{"_filename"});
                $outdir_plots = $ACTIVE_JOB{"dir_plot_images"};

                # Testen ob Verzeichnisse Leerzeichen enthalten, falls ja in " " Klammern!
                if ($outfiledata =~ /\s/) {
                        if ( !($outfiledata =~ /^\"/) ) { $outfiledata = "\"$outfiledata"; }
                        if ( !($outfiledata =~ /\"$/) ) { $outfiledata = "$outfiledata\""; }
                }
                if ($outdir_plots =~ /\s/) {
                        if ( !($outdir_plots =~ /^\"/) ) { $outdir_plots = "\"$outdir_plots"; }
                        if ( !($outdir_plots =~ /\"$/) ) { $outdir_plots = "$outdir_plots\""; }
                }
                if ($outfileraw =~ /\s/) {
                        if ( !($outfileraw =~ /^\"/) ) { $outfileraw = "\"$outfileraw"; }
                        if ( !($outfileraw =~ /\"$/) ) { $outfileraw = "$outfileraw\""; }
                }
                if ($outfileparse =~ /\s/) {
                        if ( !($outfileparse =~ /^\"/) ) { $outfileparse = "\"$outfileparse"; }
                        if ( !($outfileparse =~ /\"$/) ) { $outfileparse = "$outfileparse\""; }
                }


                # IPERF COMMAND erstellen
                if (defined($ACTIVE_JOB{"cmdline"})) {
	        	$cmdline = $ACTIVE_JOB{"cmdline"};
		} else {

                	$cmdline = $IPERF_CMD;

                        my $datarate;

                        # Client-Server: -c
                	if (defined($ACTIVE_JOB{"iperf_host"})) { $cmdline .= sprintf(" -c%s", $ACTIVE_JOB{"iperf_host"}); }

			# TIME: -t
                	if (defined($ACTIVE_JOB{"iperf_time"})) { $cmdline .= sprintf(" -t%s", $ACTIVE_JOB{"iperf_time"}); }

                        # Bandwith (only UDP!) -b
                        # INFO: NO BANDWITH ==> TCP Max. Datarate!
                        if (defined($ACTIVE_JOB{"iperf_bandwith_inc"})) {

                                my $inc_intervall = 3600;

	                        if (defined($ACTIVE_JOB{"iperf_inc_intervall"})) { $inc_intervall = $ACTIVE_JOB{"iperf_inc_intervall"}; }
                                $datarate = (sprintf("%d",floor((time - $ACTIVE_JOB{"starttime"})/$inc_intervall))) * $ACTIVE_JOB{"iperf_bandwith_inc"};
                        }
                        if (defined($ACTIVE_JOB{"iperf_bandwith"})) {
                        	$ACTIVE_JOB{"cur_datarate"} = ($datarate + $ACTIVE_JOB{"iperf_bandwith"});
                        	$cmdline .= sprintf(" -u -b%s", $ACTIVE_JOB{"cur_datarate"});
                        }

                        # Output in Bits/s, Parallel Streams = 1
			$cmdline .= " -fb -P1";
                }



		printf ("[%s] DOING JOB %s (STOPTIME: %s)\n", convert_localtime2str(time), $ACTIVE_JOB{"_filename"}, convert_localtime2str($ACTIVE_JOB{"stoptime"}));

                if (!(-r $outfileparse)) {
                      	$strHeader = sprintf ("#%s\t\t%s\t\t%s\t%s\t%s\t%s\t%s\n",  "         ", "OUTPUT", "OUPUT   ", "INPUT   ", "Jitter", "Failed Pck.", "Packets" );
                      	$strHeader .= sprintf ("#%s\t\t%s\t\t%s\t%s\t%s\t%s\t%s\n", "DATE TIME", "Bytes ", "Bits/sec", "Bits/sec", "Jitter", "Failed Pck.", "Packets" );
			$strHeader .= "#\n";
                }

                open (RAW, ">> $outfileraw");
                open (PAR, ">> $outfileparse");

		if (defined($strHeader)) {
                      	print PAR $strHeader;
                }

                # Cmdline starten und Output parsen
                if (open (PID, "$cmdline |")) {
                        printf RAW ("CMDLINE ==> %s\n", $cmdline);
                        printf RAW ("START   ==> %s\n", convert_localtime2str(time));

                        printf ("\nCMDLINE ==> %s\n", $cmdline);

                        my $cid;
                        my $tcp;
                        my $udp;
                        my $udp_inputrate;
                        my $now;

                        while ($line = <PID>) {
                                chomp ($line);

	                       	$now = convert_localtime2str(time);
                                printf RAW ("[%s] %s\n", $now, $line);

                                #DEBUG($line);

                                # FEHLER in der Verbindung - Server genuegend Zeit zum Starten geben.
                                if ($line =~ /WARNING|read failed|connect failed/i) {
					$result = 0;
                                        last;
                                }

                                if ( !(defined($tcp)) && ($line =~ /TCP port/i)) {
                                	$tcp = 1;
                                }
                                if ( !(defined($udp)) && ($line =~ /UDP port/i)) {
                                	$udp = 1;
                                }

                                if ( !(defined($cid)) && ($line =~ /^\[(\d+)\].+connected with/)) {
                                 	$cid = $1;
                                        #printf ("CID: %s\n", $cid);
                                }
                                # TCP
                                if ( (defined($tcp)) && (defined($cid)) && ($line =~ /^\[$cid\]\s*?([0-9]+\.[0-9]+)[-\s]+([0-9]+\.[0-9]+).+?sec.+?([0-9\.]+).+?Bytes.+?([0-9\.]+).+?bits\/sec/) )
                                {
					# DEBUG OUT
                                 	printf PAR ("%s\t %d\t%d\n",  $now, $3, $4);
                                 	printf ("[RESULT]\t %d\t%d\n",  $3, $4);
                                        $result = 1;
                                        undef $cid;
                                }
				# UDP Input Data Rate
                                if ( (!(defined($udp_inputrate))) && (defined($udp)) && (defined($cid)) && ($line =~ /^\[$cid\]\s*?([0-9]+\.[0-9]+)[-\s]+([0-9]+\.[0-9]+).+?sec.+?([0-9\.]+).+?Bytes.+?([0-9\.]+).+?bits\/sec/) )
                                {
					# DEBUG OUT
                                 	#printf PAR ("%s\t %d\t%d\n",  $now, $3, $4);
                                 	#printf ("[RESULT]\t %d\t%d\n",  $3, $4);
                                        $udp_inputrate = $4;
                                }

                                # UDP Output Data Rate
                                if ( (defined($udp)) && (defined($cid)) && ($line =~ /^\[$cid\]\s*?([\d|\.]+)[-\s]+([\d|\.]+).+?sec.+?([\d|\.]+).+?Bytes.+?([\d|\.]+).+?bits\/sec\s+?([\d|\.]+)\sms\s+?([\d|\.|\s]+)\/([\d|\.|\s]+)/) )
                                {
                                 	printf PAR ("%s\t %d\t%d\t%d\t%f\t%d\t%d\t%d\n", $now , $3, $4, $udp_inputrate, $5, $6, $7, $ACTIVE_JOB{"cur_datarate"});

                                 	printf ("[RESULT]\t %d\t%d\t%d\t%f\t%d\t%d\t%d\n", $3, $4, $udp_inputrate, $5, $6, $7,$ACTIVE_JOB{"cur_datarate"});
                                        $result = 1;
                                        undef $cid;
                                        undef $udp_inputrate;
                                }

                        }

                 	close(PID);
                        printf ("\n");
	                printf RAW ("STOP   ==> %s\n", convert_localtime2str(time));

                }


                close (PAR);
                close (RAW);


                if (defined($ACTIVE_JOB{"parse"}) && $ACTIVE_JOB{"parse"} > 0) {
	                # Output Parsen
	                if (!(defined($PARSE_LASTTIME))) { $PARSE_LASTTIME = time; }
	                #printf ("Parse Time: %s > %s\n",  (time - $PARSE_LASTTIME), $PARSE_TIME);
	                if ( ((time - $PARSE_LASTTIME) > $PARSE_TIME) || (($ACTIVE_JOB{"stoptime"}-time) < 60) )  {

	                        printf ("PARSE_CMD: %s\n\n",  "$PARSE_CMD $outfileparse $outfiledata");
	                        # RAW Data fr gnuplot aufbereiten
	                        if (-r $PARSE_CMD) {
	                                system("$PARSE_CMD $outfileparse $outfiledata");
	                        } else { DEBUG("Can't find $PARSE_CMD!"); }


                                if ( defined($ACTIVE_JOB{"plot"}) && $ACTIVE_JOB{"plot"} > 0) {

	                                # Daten mit gnuplot verarbeiten
	                                # Script fuer gnuplot: generatePlot.pl DATFILE OUTDIR DATE TITLE
	                                my $plot_date = sprintf("%s-%s-%s", $date[5]+1900, $date[4]+1, $date[3]);
	                                my $plot_title = "\"TCP Max. Input Datarate\"";
	                                # Check if UDP Test...

	                                if ( $cmdline =~ /-b\s*?(\d+)([k|m|g|K|M|G])/ ) {
	                                        $plot_title = "\"UDP Input Datarate $1";
	                                        if (defined($2)) {
	                                                if ($2 eq 'm') { $plot_title .= " MBit/s\""; }
	                                                if ($2 eq 'k') { $plot_title .=" KBit/s\""; }
	                                        }
	                                }
	                                if (-r $PLOT_CMD) {
                                        	printf ("PLOT_CMD %s\n\n", "$PLOT_CMD $outfiledata $outdir_plots $plot_date $plot_title");
	                                        system("$PLOT_CMD $outfiledata $outdir_plots $plot_date $plot_title");
	                                } else { DEBUG("Can't find $PLOT_CMD!"); }
				}

	                        $PARSE_LASTTIME = time;
                	}
                }

                # Kein Ergebnis erhalten -> Fehler!
                if ( $result < 1 ) {
                	# Sleep wg. Fehler -> Backoff Timer...
	                printf ("ERROR ==> Going sleep for %d seconds.\n", $sleep_error);
	                sleep($sleep_error);
                        if ($sleep_error <= $sleep_error_max) {
                        	$sleep_error += $sleep_error_backoff;
                        } else {
                        	$sleep_error = $sleep_error_start;
                        }
                } else {
                	# sleep_error zurueck setzen, da Verbindung erfolgreich
                          $sleep_error = $sleep_error_start;

                	# Sleeptime Cmd ausfuehren (siehe job config)
	                if (defined($ACTIVE_JOB{"sleeptime"})) {
	                        printf ("[%s] DOING SLEEP ... (%d seconds)\n", convert_localtime2str(time), $ACTIVE_JOB{"sleeptime"});
	                        sleep($ACTIVE_JOB{"sleeptime"});
	                }
                }

        } # END IF

} # END SUB

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #

sub read_jobs {

	my $now = time;

        my @tmpFiles;
        my @files;
        my $filename;
        my $line;

	my @tmp_jobs;
#        my %TIMELINE;
        my $left;
        my $right;

        opendir (DIR, "$JOBDIR") or die "can't open directory $JOBDIR!\n";
        @files = grep { /^./ && -f "$JOBDIR/$_"} readdir(DIR);
        closedir(DIR);

        foreach $filename (@files) {
                if ($filename =~ /^BREAK$/) {
                	$break_sig = 1;
                        return 0;
                }
                if ($filename =~ /^JOB|^job/) {
                 	push(@tmpFiles, $filename);
                }
        }

        # Dateien "job*" verarbeiten...

        chdir($JOBDIR);
	foreach $filename (@tmpFiles) {

		my %CONFIG;

         	# Dateien einlesen und verarbeiten
		if (open (IN, "< $filename")) {
                	#printf("Processing $filename...\n");
                 	while ($line = <IN>) {
				chomp ($line);

                                # Kommentar Zeilen berspringen # oder Leerzeichen
                                if ($line =~ /^[#|\s]/) { next;}

                                # Konfigurationen suchen und laden.
                                if ($line =~ /^(iperf_host|iperf_time|iperf_inc_intervall)=(.*)/i) {
                                	$left = lc($1);
                                        $right = $2;
                                        $CONFIG{$left} = $right;
                                }
                                if ($line =~ /^(iperf_bandwith|iperf_bandwith_inc)=(.*)/i) {
                                	$left = lc($1);
                                        $right = $2;

                                        if ($right =~ /(\d+)([mk])$/) {
                                        	if ($2 eq 'k') {$right = $1 * 1000; }
                                                if ($2 eq 'm') {$right = $1 * 1000000; }
                                        }

                                        $CONFIG{$left} = $right;
                                }

                                if ($line =~ /^(plot|parse)=(.*)$/i) {
                                	$left = lc($1);
                                        $right = $2;
					$CONFIG{$left} = $right;
                                }

                                if ($line =~ /^(cmdline|starttime|job_duration|stoptime|sleeptime)=(.*)$/i) {
                                	$left = lc($1);
                                        $right = $2;

 					if ($left =~ /starttime|stoptime/) {
                                        	$right = convert_str2localtime($right);
                                        }

                                        if ($left =~ /job_duration/) {
                                        	$CONFIG{"stoptime"} = ($CONFIG{"starttime"})+ $right;
                                        }
      					$CONFIG{$left} = $right;

                                        next;
                                }
                                if ($line =~ /^(dir_raw|dir_parse|dir_plot_data|dir_plot_images)=(.*)$/i) {
                                	$left = lc($1);
                                        $right = $2;
                                        $right =~ s/\"//g;
                                        $CONFIG{$left} = $right;
                                        next;
                                }

                        } # END WHILE
                        close (IN);

                        # Testen der Stop-Zeiten
                        # Wenn in Vergangenheit, job als DONE_ markieren
                        if ($now < $CONFIG{"stoptime"}) {
                        	$CONFIG{"_filename"} = $filename;
	                        push(@tmp_jobs, \%CONFIG);
                        } else {
				rename($filename, ("DONE_".$now."_".$filename));
                        }

                }

        }

        if ($#tmp_jobs >= 0 ) {
		@jobs = @tmp_jobs;
        	return 1;
        }
	else {	return 0; }


} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub get_next_job {

	my %CONFIG;
        my $tmpTime; #=999999999999;
        my $tmp_id; #=-1;
        my $i;

        for ($i=0; $i <= $#jobs; $i++)
        {
		%CONFIG = %{$jobs[$i]};
		if ($tmpTime > $CONFIG{"starttime"} or ($i == 0)) {

                	$tmpTime = $CONFIG{"starttime"};
                	$tmp_id = $i;
                }
        }
        if ($tmp_id >=0) {
		%CONFIG = %{$jobs[$tmp_id]};
        	return %CONFIG;
        }
#        else {
                # NUR IM FEHLERFALL !!
#                return %{$jobs[$tmp_id]};
#        }

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub print_job_details {

	my %CONFIG;
        my @keylist;
        my $i=0;

        printf("-----------------------------------------\n");
        printf("Jobs in Queue %d:\n", $#jobs+1);
        printf("-----------------------------------------\n");

        foreach my $j (@jobs) {
                	%CONFIG = %{$j};
                @keylist = sort keys %CONFIG;
                foreach my $key (@keylist) {
                	if ($key =~ /^(starttime|stoptime)/) {
				printf("%03d %-20s ==> %-50s\n", $i,$key, convert_localtime2str($CONFIG{$key}) );
                        } else {
	                	printf("%03d %-20s ==> %-50s\n", $i,$key, $CONFIG{$key} );
                        }
                }
                printf("-----------------------------------------\n");
                $i++;
        }
        printf("Current Timeticks: %d (%s)\n", time, convert_localtime2str(time));


} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub convert_str2localtime {

	my $ticks;
        my $str = $_[0];
        $str =~ s/-|://g;
	$str =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/;
        my $year = $1-1900;
        my $month = $2-1;
        my $mday = $3;
        my $hour = $4;
        my $min = $5;
        my $sec = 0;

	$ticks = timelocal($sec, $min, $hour, $mday, $month, $year);
        return $ticks;

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub convert_localtime2str {
 	my @date = localtime(time);
        my $str;

      	if (defined $_[0]) { @date = localtime($_[0])}

        $str = sprintf("%d%02d%02d %02d:%02d:%02d", (1900+$date[5]), $date[4]+1, $date[3], $date[2], $date[1], $date[0]);

        return $str;

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub get_date() {
	my @date = localtime(time);
        return (sprintf("%4d%02d%02d", $date[5]+1900, $date[4]+1, $date[3]));

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub get_dateTime() {
        return convert_localtime2str(time);
#        return (sprintf("%4d%02d%02d %02d:%02d:%02d", $date[5]+1900, $date[4]+1, $date[3], $date[2], $date[1], $date[0]));

} # END SUB
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
sub DEBUG {
	printf ("[%s DEBUG]: %s\n", convert_localtime2str(time), $_[0]);
}
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #